package com.qmth.ops.biz.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.ops.biz.dao.DeployDao;
import com.qmth.ops.biz.domain.Deploy;
import com.qmth.ops.biz.query.DeployQuery;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.UUID;

@Service
public class DeployService extends ServiceImpl<DeployDao, Deploy> {

    @Resource
    private DeployDao deployDao;

    @Transactional
    public Deploy insert(Deploy deploy) {
        deploy.setAccessKey(UUID.randomUUID().toString().replaceAll("-", ""));
        deploy.setAccessSecret(RandomStringUtils.random(32, true, true));
        deploy.setCreateTime(System.currentTimeMillis());
        deploy.setUpdateTime(deploy.getCreateTime());
        deployDao.insert(deploy);
        return deploy;
    }

    @Transactional
    public Deploy update(Deploy deploy) {
        deployDao.update(deploy,
                new LambdaUpdateWrapper<Deploy>().set(deploy.getName() != null, Deploy::getName, deploy.getName())
                        .set(deploy.getMode() != null, Deploy::getMode, deploy.getMode())
                        .set(deploy.getControl() != null, Deploy::getControl, deploy.getControl())
                        .set(deploy.getIpAllow() != null, Deploy::getIpAllow, deploy.getIpAllow())
                        .set(Deploy::getUpdateTime, System.currentTimeMillis()).eq(Deploy::getId, deploy.getId()));
        return deployDao.selectById(deploy.getId());
    }

    public DeployQuery query(DeployQuery query) {
        return deployDao.selectPage(query, query.build());
    }

    public Deploy findByAccessKey(String accessKey) {
        return deployDao.selectOne(new LambdaQueryWrapper<Deploy>().eq(Deploy::getAccessKey, accessKey));
    }

    public Deploy findById(Long id) {
        return deployDao.selectById(id);
    }

}

